<?php
/**
* optimizeMember's URI protection routines *( for current URI )*.
*
* Copyright: © 2009-2011
* {@link http://www.optimizepress.com/ optimizePress, Inc.}
* ( coded in the USA )
*
* Released under the terms of the GNU General Public License.
* You should have received a copy of the GNU General Public License,
* along with this software. In the main directory, see: /licensing/
* If not, see: {@link http://www.gnu.org/licenses/}.
*
* @package optimizeMember\URIs
* @since 3.5
*/
if (realpath (__FILE__) === realpath ($_SERVER["SCRIPT_FILENAME"]))
	exit ("Do not access this file directly.");
/**/
if (!class_exists ("c_ws_plugin__optimizemember_ruris"))
	{
		/**
		* optimizeMember's URI protection routines *( for current URI )*.
		*
		* @package optimizeMember\URIs
		* @since 3.5
		*/
		class c_ws_plugin__optimizemember_ruris
			{
				/**
				* Handles URI Level Access permissions *( for current URI )*.
				*
				* @package optimizeMember\URIs
				* @since 3.5
				*
				* @return null Or exits script execution after redirection.
				*/
				public static function check_ruri_level_access ()
					{
						do_action ("ws_plugin__optimizemember_before_check_ruri_level_access", get_defined_vars ());
						/**/
						$excluded = apply_filters ("ws_plugin__optimizemember_check_ruri_level_access_excluded", false, get_defined_vars ());
						/**/
						if (!$excluded && $GLOBALS["WS_PLUGIN__"]["optimizemember"]["o"]["membership_options_page"]) /* Has it been excluded? */
							{
								if (!c_ws_plugin__optimizemember_systematics::is_wp_systematic_use_page ()) /* Do NOT touch WordPress Systematics. This excludes all WordPress Systematics. */
									{
										$user = (is_user_logged_in () && is_object ($user = wp_get_current_user ()) && !empty ($user->ID)) ? $user : false; /* Current User's object. */
										/**/
										if ($GLOBALS["WS_PLUGIN__"]["optimizemember"]["o"]["login_redirection_override"] && ($login_redirection_uri = c_ws_plugin__optimizemember_login_redirects::login_redirection_uri ($user, "root-returns-false")) && preg_match ("/^" . preg_quote ($login_redirection_uri, "/") . "$/", $_SERVER["REQUEST_URI"]) && c_ws_plugin__optimizemember_no_cache::no_cache_constants (true) && (!$user || !$user->has_cap ("access_optimizemember_level0")))
											c_ws_plugin__optimizemember_mo_page::wp_redirect_w_mop_vars /* Configure MOP Vars here. */ ("ruri", $_SERVER["REQUEST_URI"], "level", 0, $_SERVER["REQUEST_URI"], "sys") . exit ();
										/**/
										else if (!c_ws_plugin__optimizemember_systematics::is_systematic_use_page ()) /* Do NOT protect Systematics. However, there is 1 exception above ^. */
											{
												for ($n = $GLOBALS["WS_PLUGIN__"]["optimizemember"]["c"]["levels"]; $n >= 0; $n--) /* URIs. Go through each Level. */
													{
														if ($GLOBALS["WS_PLUGIN__"]["optimizemember"]["o"]["level" . $n . "_ruris"]) /* URIs configured at this Level? */
															/**/
															foreach (preg_split ("/[\r\n\t]+/", c_ws_plugin__optimizemember_ruris::fill_ruri_level_access_rc_vars ($GLOBALS["WS_PLUGIN__"]["optimizemember"]["o"]["level" . $n . "_ruris"], $user)) as $str)
																if ($str && preg_match ("/" . preg_quote ($str, "/") . "/", $_SERVER["REQUEST_URI"]) && c_ws_plugin__optimizemember_no_cache::no_cache_constants (true) && (!$user || !$user->has_cap ("access_optimizemember_level" . $n)))
																	c_ws_plugin__optimizemember_mo_page::wp_redirect_w_mop_vars /* Configure MOP Vars here. */ ("ruri", $_SERVER["REQUEST_URI"], "level", $n, $_SERVER["REQUEST_URI"]) . exit ();
													}
											}
										/**/
										do_action ("ws_plugin__optimizemember_during_check_ruri_level_access", get_defined_vars ());
									}
							}
						/**/
						do_action ("ws_plugin__optimizemember_after_check_ruri_level_access", get_defined_vars ());
						/**/
						return; /* For uniformity. */
					}
				/**
				* Fills Replacement Code variables in URIs; collectively.
				*
				* @package optimizeMember\URIs
				* @since 3.5
				*
				* @param str $uris A URI string, or a string of multiple URIs is also fine.
				* @param obj $user Optional. A `WP_User` object. Defaults to the current User, if logged-in.
				* @return str Collective string of input URIs, with Replacement Codes having been filled.
				*/
				public static function fill_ruri_level_access_rc_vars ($uris = FALSE, $user = FALSE)
					{
						eval ('foreach(array_keys(get_defined_vars())as$__v)$__refs[$__v]=&$$__v;');
						do_action ("ws_plugin__optimizemember_before_fill_ruri_level_access_rc_vars", get_defined_vars ());
						unset ($__refs, $__v); /* Unset defined __refs, __v. */
						/**/
						$uris = (string)$uris; /* Force ``$uris`` to a string value. */
						$orig_uris = $uris; /* Record the original URIs that were passed in; collectively. */
						/**/
						$user = ((is_object ($user) || is_object ($user = (is_user_logged_in ()) ? wp_get_current_user () : false)) && !empty ($user->ID)) ? $user : false;
						/**/
						$user_id = ($user) ? (string)$user->ID : "";
						$user_login = ($user) ? (string)strtolower ($user->user_login) : "";
						/**/
						$user_level = (string)c_ws_plugin__optimizemember_user_access::user_access_level ($user);
						$user_role = (string)c_ws_plugin__optimizemember_user_access::user_access_role ($user);
						$user_ccaps = (string)implode ("-", c_ws_plugin__optimizemember_user_access::user_access_ccaps ($user));
						$user_logins = ($user) ? (string)(int)get_user_option ("optimizemember_login_counter", $user_id) : "-1";
						/**/
						$uris = (strlen ($user_login)) ? preg_replace ("/%%current_user_login%%/i", c_ws_plugin__optimizemember_utils_strings::esc_ds ($user_login), $uris) : $uris;
						$uris = (strlen ($user_id)) ? preg_replace ("/%%current_user_id%%/i", c_ws_plugin__optimizemember_utils_strings::esc_ds ($user_id), $uris) : $uris;
						$uris = (strlen ($user_level)) ? preg_replace ("/%%current_user_level%%/i", c_ws_plugin__optimizemember_utils_strings::esc_ds ($user_level), $uris) : $uris;
						$uris = (strlen ($user_role)) ? preg_replace ("/%%current_user_role%%/i", c_ws_plugin__optimizemember_utils_strings::esc_ds ($user_role), $uris) : $uris;
						$uris = (strlen ($user_ccaps)) ? preg_replace ("/%%current_user_ccaps%%/i", c_ws_plugin__optimizemember_utils_strings::esc_ds ($user_ccaps), $uris) : $uris;
						$uris = (strlen ($user_logins)) ? preg_replace ("/%%current_user_logins%%/i", c_ws_plugin__optimizemember_utils_strings::esc_ds ($user_logins), $uris) : $uris;
						/**/
						return apply_filters ("ws_plugin__optimizemember_fill_ruri_level_access_rc_vars", $uris, get_defined_vars ());
					}
			}
	}
?>